@@ -79,6 +79,13 @@ class ElementBase:
     def getVramOffset(self, localOffset: int) -> int:
         return self.vram + localOffset
 
+    def containsVram(self, vram: int) -> bool:
+        if vram < self.vram:
+            return False
+        if vram >= self.vramEnd:
+            return False
+        return True
+
 
     def getLabelFromSymbol(self, sym: ContextSymbol|None) -> str:
         "Generates a glabel for the passed symbol, including an optional index value if it was set and it is enabled in the GlobalConfig"
@@ -132,6 +139,12 @@ class ElementBase:
         return self.context.globalSegment
 
     def getSegmentForVram(self, vram: int) -> SymbolsSegment:
+        if self._ownSegmentReference is None:
+            if self.context.globalSegment.isVromInRange(self.vromStart):
+                self._ownSegmentReference = self.context.globalSegment
+        if self.context.globalSegment.isVramInRange(vram):
+            return self.context.globalSegment
+
         if self.overlayCategory is not None:
             # If this element is part of an overlay segment
 
@@ -146,14 +159,15 @@ class ElementBase:
                     if overlaySegment.isVramInRange(vram):
                         return overlaySegment
 
-        if self._ownSegmentReference is None:
-            if self.context.globalSegment.isVromInRange(self.vromStart):
-                self._ownSegmentReference = self.context.globalSegment
-        if self.context.globalSegment.isVramInRange(vram):
-            return self.context.globalSegment
         return self.context.unknownSegment
 
     def getSegmentForVrom(self, vrom: int) -> SymbolsSegment:
+        if self._ownSegmentReference is None:
+            if self.context.globalSegment.isVromInRange(self.vromStart):
+                self._ownSegmentReference = self.context.globalSegment
+        if self.context.globalSegment.isVromInRange(vrom):
+            return self.context.globalSegment
+
         if self.overlayCategory is not None:
             # If this element is part of an overlay segment
 
@@ -177,17 +191,17 @@ class ElementBase:
                         if overlaySegment.isVromInRange(vrom):
                             return overlaySegment
 
-        if self._ownSegmentReference is None:
-            if self.context.globalSegment.isVromInRange(self.vromStart):
-                self._ownSegmentReference = self.context.globalSegment
-        if self.context.globalSegment.isVromInRange(vrom):
-            return self.context.globalSegment
         return self.context.unknownSegment
 
 
-    def getSymbol(self, vramAddress: int, tryPlusOffset: bool = True, checkUpperLimit: bool = True) -> ContextSymbol|None:
+    def getSymbol(self, vramAddress: int, tryPlusOffset: bool = True, checkUpperLimit: bool = True, checkGlobalSegment: bool = True) -> ContextSymbol|None:
         "Searches symbol or a symbol with an addend if `tryPlusOffset` is True"
 
+        if self.overlayCategory is None or checkGlobalSegment:
+            contextSym = self.context.globalSegment.getSymbol(vramAddress, tryPlusOffset=tryPlusOffset, checkUpperLimit=checkUpperLimit)
+            if contextSym is not None:
+                return contextSym
+
         if self.overlayCategory is not None:
             # If this element is part of an overlay segment
 
@@ -205,20 +219,22 @@ class ElementBase:
             for overlayCategory, segmentsPerVrom in self.context.overlaySegments.items():
                 if self.overlayCategory != overlayCategory:
                     for overlaySegment in segmentsPerVrom.values():
-                        # if overlaySegment.isVramInRange(vramAddress):
+                        if not overlaySegment.isVramInRange(vramAddress):
+                            continue
                         contextSym = overlaySegment.getSymbol(vramAddress, tryPlusOffset=tryPlusOffset, checkUpperLimit=checkUpperLimit)
                         if contextSym is not None:
                             return contextSym
 
-        # if self.context.globalSegment.isVramInRange(vramAddress):
-        contextSym = self.context.globalSegment.getSymbol(vramAddress, tryPlusOffset=tryPlusOffset, checkUpperLimit=checkUpperLimit)
-        if contextSym is not None:
-            return contextSym
+        if not checkGlobalSegment:
+            return None
+
         contextSym = self.context.unknownSegment.getSymbol(vramAddress, tryPlusOffset=tryPlusOffset, checkUpperLimit=checkUpperLimit)
         if self._ownSegmentReference is not None:
             if contextSym is not None and contextSym.vromAddress is not None:
                 if not self._ownSegmentReference.isVromInRange(contextSym.getVrom()):
                     return None
+        if not GlobalConfig.ALLOW_UNKSEGMENT:
+            return None
         return contextSym
 
     def getSymbolByVrom(self, vromAddress: int, tryPlusOffset: bool = True, checkUpperLimit: bool = True) -> ContextSymbol|None:
@@ -300,7 +316,7 @@ class ElementBase:
 
     def canUseAddendsOnData(self) -> bool:
         segment = self.getSegmentForVram(self.vram)
-        return self.vram in segment.dataSymbolsWithReferencesWithAddends
+        return GlobalConfig.ALLOW_ALL_ADDENDS_ON_DATA or self.vram in segment.dataSymbolsWithReferencesWithAddends
 
     def canUseConstantsOnData(self) -> bool:
         segment = self.getSegmentForVram(self.vram)
