@@ -7,9 +7,10 @@ from __future__ import annotations
 
 import argparse
 import csv
-import os
 import hashlib
 import json
+from pathlib import Path
+import rabbitizer
 import struct
 import subprocess
 import sys
@@ -45,28 +46,28 @@ def isStdoutRedirected() -> bool:
 def getStrHash(byte_array: bytearray) -> str:
     return str(hashlib.md5(byte_array).hexdigest())
 
-def writeBytearrayToFile(filepath: str, array_of_bytes: bytearray):
-    with open(filepath, mode="wb") as f:
+def writeBytearrayToFile(filepath: Path, array_of_bytes: bytearray):
+    with filepath.open(mode="wb") as f:
         f.write(array_of_bytes)
 
-def readFileAsBytearray(filepath: str) -> bytearray:
-    if not os.path.exists(filepath):
+def readFileAsBytearray(filepath: Path) -> bytearray:
+    if not filepath.exists():
         return bytearray(0)
-    with open(filepath, mode="rb") as f:
+    with filepath.open(mode="rb") as f:
         return bytearray(f.read())
 
-def readFile(filepath: str) -> list[str]:
-    with open(filepath) as f:
+def readFile(filepath: Path) -> list[str]:
+    with filepath.open() as f:
         return [x.strip() for x in f.readlines()]
 
-def readJson(filepath):
-    with open(filepath) as f:
+def readJson(filepath: Path):
+    with filepath.open() as f:
         return json.load(f)
 
 def removeExtraWhitespace(line: str) -> str:
     return " ".join(line.split())
 
-def bytesToBEWords(array_of_bytes: bytearray, offset: int=0, offsetEnd: int|None=None) -> list[int]:
+def endianessBytesToWords(endian: InputEndian, array_of_bytes: bytearray, offset: int=0, offsetEnd: int|None=None) -> list[int]:
     totalBytesCount = len(array_of_bytes)
     if totalBytesCount == 0:
         return list()
@@ -76,7 +77,7 @@ def bytesToBEWords(array_of_bytes: bytearray, offset: int=0, offsetEnd: int|None
         bytesCount = offsetEnd
     bytesCount -= offset
 
-    if GlobalConfig.ENDIAN == InputEndian.MIDDLE:
+    if endian == InputEndian.MIDDLE:
         # Convert middle endian to big endian
         halfwords = bytesCount//2
         little_byte_format = f"<{halfwords}H"
@@ -86,23 +87,40 @@ def bytesToBEWords(array_of_bytes: bytearray, offset: int=0, offsetEnd: int|None
 
     words = bytesCount//4
     endian_format = f">{words}I"
-    if GlobalConfig.ENDIAN == InputEndian.LITTLE:
+    if endian == InputEndian.LITTLE:
         endian_format = f"<{words}I"
     return list(struct.unpack_from(endian_format, array_of_bytes, offset))
 
-def beWordsToBytes(words_list: list[int], buffer: bytearray) -> bytearray:
+def bytesToWords(array_of_bytes: bytearray, offset: int=0, offsetEnd: int|None=None) -> list[int]:
+    return endianessBytesToWords(GlobalConfig.ENDIAN, array_of_bytes, offset, offsetEnd)
+
+#! deprecated
+bytesToBEWords = bytesToWords
+
+def endianessWordsToBytes(endian: InputEndian, words_list: list[int], buffer: bytearray) -> bytearray:
+    if endian == InputEndian.MIDDLE:
+        raise BufferError("TODO: wordsToBytesEndianess: GlobalConfig.ENDIAN == InputEndian.MIDDLE")
+
     words = len(words_list)
-    big_endian_format = f">{words}I"
-    struct.pack_into(big_endian_format, buffer, 0, *words_list)
+    endian_format = f">{words}I"
+    if endian == InputEndian.LITTLE:
+        endian_format = f"<{words}I"
+    struct.pack_into(endian_format, buffer, 0, *words_list)
     return buffer
 
+def wordsToBytes(words_list: list[int], buffer: bytearray) -> bytearray:
+    return endianessWordsToBytes(GlobalConfig.ENDIAN, words_list, buffer)
+
+#! deprecated
+beWordsToBytes = wordsToBytes
+
 def wordToFloat(word: int) -> float:
     return struct.unpack('>f', struct.pack('>I', word))[0]
 
 def qwordToDouble(qword: int) -> float:
     return struct.unpack('>d', struct.pack('>Q', qword))[0]
 
-def beWordToCurrenEndian(word: int) -> int:
+def wordToCurrenEndian(word: int) -> int:
     if GlobalConfig.ENDIAN == InputEndian.BIG:
         return word
 
@@ -120,9 +138,10 @@ def runCommandGetOutput(command: str, args: list[str]) -> list[str] | None:
     except:
         return None
 
-def readCsv(filepath: str) -> list[list[str]]:
+def readCsv(filepath: Path) -> list[list[str]]:
     data: list[list[str]] = []
-    with open(filepath) as f:
+
+    with filepath.open() as f:
         lines = f.readlines()
         processedLines = [x.strip().split("#")[0] for x in lines]
         csvReader = csv.reader(processedLines)
@@ -131,55 +150,68 @@ def readCsv(filepath: str) -> list[list[str]]:
 
     return data
 
-def decodeString(buf: bytearray, offset: int) -> tuple[str, int]:
-    # Escape characters that are unlikely to be used
-    bannedEscapeCharacters = [
-        0x01,
-        0x02,
-        0x03,
-        0x04,
-        0x05,
-        0x06,
-        # 0x07, # '\a'
-        0x08, # '\b'
-        # 0x09, # '\t'
-        # 0x0A, # '\n'
-        0x0B, # '\v'
-        # 0x0C, # '\f'
-        # 0x0D, # '\r'
-        0x0E,
-        0x0F,
-        0x10,
-        0x11,
-        0x12,
-        0x13,
-        0x14,
-        0x15,
-        0x16,
-        0x17,
-        0x18,
-        0x19,
-        0x1A,
-        # 0x1B, # VT escape sequences
-        0x1C,
-        0x1D,
-        0x1E,
-        0x1F,
-    ]
+# Escape characters that are unlikely to be used
+bannedEscapeCharacters = {
+    0x01,
+    0x02,
+    0x03,
+    0x04,
+    0x05,
+    0x06,
+    0x07, # '\a'
+    0x08, # '\b'
+    # 0x09, # '\t'
+    # 0x0A, # '\n'
+    0x0B, # '\v'
+    # 0x0C, # '\f'
+    # 0x0D, # '\r'
+    0x0E,
+    0x0F,
+    0x10,
+    0x11,
+    0x12,
+    0x13,
+    0x14,
+    0x15,
+    0x16,
+    0x17,
+    0x18,
+    0x19,
+    0x1A,
+    # 0x1B, # VT escape sequences
+    0x1C,
+    0x1D,
+    0x1E,
+    0x1F,
+}
+
+escapeCharactersSpecialCases = {0x1B, 0x8C, 0x8D}
+
+def decodeString(buf: bytearray, offset: int, stringEncoding: str) -> tuple[list[str], int]:
+    result = []
+
     dst = bytearray()
     i = 0
     while offset + i < len(buf) and buf[offset + i] != 0:
-        dst.append(buf[offset + i])
-        i += 1
-    if offset + i > len(buf):
-        # We reached the end of the buffer without reaching a 0.
-        raise RuntimeError()
-
-    for bannedChar in bannedEscapeCharacters:
-        if bannedChar in dst:
+        char = buf[offset + i]
+        if char in bannedEscapeCharacters:
             raise RuntimeError()
+        elif char in escapeCharactersSpecialCases:
+            if dst:
+                decoded = rabbitizer.Utils.escapeString(dst.decode(stringEncoding))
+                result.append(decoded)
+                dst.clear()
+            result.append(f"\\x{char:02X}")
+        else:
+            dst.append(char)
+        i += 1
 
-    result = dst.decode("EUC-JP").replace("\n", "\\n").replace("\r", "\\r").replace("\t", "\\t").replace('"', '\\"').replace("\f", "\\f").replace("\a", "\\a").replace("\x1B", "\\x1B")
+    if offset + i > len(buf):
+        raise RuntimeError("Reached the end of the buffer without finding an 0")
+
+    if dst:
+        decoded = rabbitizer.Utils.escapeString(dst.decode(stringEncoding))
+        result.append(decoded)
     return result, i
 
 
