@@ -8,6 +8,8 @@ from __future__ import annotations
 import dataclasses
 import struct
 
+from .. import common
+
 
 # a.k.a. Sym (symbol)
 @dataclasses.dataclass
@@ -30,11 +32,15 @@ class Elf32SymEntry:
 
     @staticmethod
     def fromBytearray(array_of_bytes: bytearray, offset: int = 0) -> Elf32SymEntry:
-        entryFormat = ">IIIBBH"
+        entryFormat = common.GlobalConfig.ENDIAN.toFormatString() + "IIIBBH"
         unpacked = struct.unpack_from(entryFormat, array_of_bytes, offset)
 
         return Elf32SymEntry(*unpacked)
 
+    @staticmethod
+    def structSize() -> int:
+        return 0x10
+
 
 class Elf32Syms:
     def __init__(self, array_of_bytes: bytearray, offset: int, rawSize: int):
@@ -42,9 +48,16 @@ class Elf32Syms:
         self.offset: int = offset
         self.rawSize: int = rawSize
 
-        for i in range(rawSize // 0x10):
-            entry = Elf32SymEntry.fromBytearray(array_of_bytes, offset + i*0x10)
+        for i in range(rawSize // Elf32SymEntry.structSize()):
+            entry = Elf32SymEntry.fromBytearray(array_of_bytes, offset + i*Elf32SymEntry.structSize())
             self.symbols.append(entry)
 
     def __getitem__(self, key: int) -> Elf32SymEntry:
         return self.symbols[key]
+
+    def __iter__(self):
+        for entry in self.symbols:
+            yield entry
+
+    def __len__(self) -> int:
+        return len(self.symbols)
