@@ -5,6 +5,8 @@
 
 from __future__ import annotations
 
+import rabbitizer
+
 from ... import common
 
 from .. import symbols
@@ -14,10 +16,16 @@ from . import SectionBase
 
 class SectionRodata(SectionBase):
     def __init__(self, context: common.Context, vromStart: int, vromEnd: int, vram: int, filename: str, array_of_bytes: bytearray, segmentVromStart: int, overlayCategory: str|None):
-        super().__init__(context, vromStart, vromEnd, vram, filename, array_of_bytes, common.FileSectionType.Rodata, segmentVromStart, overlayCategory)
+        if common.GlobalConfig.ENDIAN_RODATA is not None:
+            words = common.Utils.endianessBytesToWords(common.GlobalConfig.ENDIAN_RODATA, array_of_bytes, vromStart, vromEnd)
+        else:
+            words = common.Utils.bytesToWords(array_of_bytes, vromStart, vromEnd)
+        super().__init__(context, vromStart, vromEnd, vram, filename, words, common.FileSectionType.Rodata, segmentVromStart, overlayCategory)
 
         self.bytes: bytearray = bytearray(self.sizew*4)
-        common.Utils.beWordsToBytes(self.words, self.bytes)
+        common.Utils.wordsToBytes(self.words, self.bytes)
+
+        self.stringEncoding: str = "EUC-JP"
 
 
     def _stringGuesser(self, contextSym: common.ContextSymbol, localOffset: int) -> bool:
@@ -35,7 +43,21 @@ class SectionRodata(SectionBase):
             return False
 
         try:
-            common.Utils.decodeString(self.bytes, localOffset)
+            _, rawStringSize = common.Utils.decodeString(self.bytes, localOffset, self.stringEncoding)
+
+            # Check if there is already another symbol after the current one and before the end of the string,
+            # in which case we say this symbol should not be a string
+            otherSym = self.getSymbol(self.getVramOffset(localOffset) + rawStringSize, checkUpperLimit=False, checkGlobalSegment=False)
+            if otherSym != contextSym:
+                return False
+
+            # To be a valid aligned string, the next word-aligned bytes needs to be zero
+            checkStartOffset = localOffset + rawStringSize
+            checkEndOffset = min((checkStartOffset & ~3) + 4, len(self.bytes))
+            while checkStartOffset < checkEndOffset:
+                if self.bytes[checkStartOffset] != 0:
+                    return False
+                checkStartOffset += 1
         except (UnicodeDecodeError, RuntimeError):
             # String can't be decoded
             return False
@@ -52,7 +74,7 @@ class SectionRodata(SectionBase):
             if relocSymbol is not None:
                 if relocSymbol.name is not None and relocSymbol.name.startswith("."):
                     sectType = common.FileSectionType.fromStr(relocSymbol.name)
-                    relocSymbol.sectionType = sectType
+                    # relocSymbol.sectionType = sectType
 
                     relocName = f"{relocSymbol.name}_{w:06X}"
                     contextOffsetSym = common.ContextOffsetSymbol(w, relocName, sectType)
@@ -60,7 +82,8 @@ class SectionRodata(SectionBase):
                         # jumptable
                         relocName = f"L{w:06X}"
                         contextOffsetSym = self.context.addOffsetJumpTableLabel(w, relocName, common.FileSectionType.Text)
-                        relocSymbol.type = contextOffsetSym.type
+                        if contextOffsetSym.type == common.SymbolSpecialType.jumptablelabel:
+                            relocSymbol.jumptableLabel = True
                         offsetSym = self.context.getOffsetSymbol(inFileOffset, self.sectionType)
                         if offsetSym is not None:
                             offsetSym.type = common.SymbolSpecialType.jumptable
@@ -75,11 +98,16 @@ class SectionRodata(SectionBase):
         symbolList = []
         localOffset = 0
 
+        lastVramSymbol: common.ContextSymbol | None = None
+
         partOfJumpTable = False
         for w in self.words:
             currentVram = self.getVramOffset(localOffset)
             contextSym = self.getSymbol(currentVram, tryPlusOffset=False)
 
+            if contextSym is not None:
+                lastVramSymbol = contextSym
+
             if contextSym is not None and contextSym.isJumpTable():
                 partOfJumpTable = True
 
@@ -87,25 +115,33 @@ class SectionRodata(SectionBase):
                 if localOffset in self.pointersOffsets:
                     partOfJumpTable = True
 
+                if w == 0:
+                    partOfJumpTable = False
+
                 elif contextSym is not None:
                     partOfJumpTable = False
 
                 elif ((w >> 24) & 0xFF) != 0x80:
                     partOfJumpTable = False
+                    if lastVramSymbol is not None and lastVramSymbol.isJumpTable() and lastVramSymbol.isGot and common.GlobalConfig.GP_VALUE is not None:
+                        partOfJumpTable = True
 
             if partOfJumpTable:
-                labelSym = self.addJumpTableLabel(w, isAutogenerated=True)
+                if lastVramSymbol is not None and lastVramSymbol.isGot and common.GlobalConfig.GP_VALUE is not None:
+                    labelAddr = common.GlobalConfig.GP_VALUE + rabbitizer.Utils.from2Complement(w, 32)
+                    labelSym = self.addJumpTableLabel(labelAddr, isAutogenerated=True)
+                else:
+                    labelSym = self.addJumpTableLabel(w, isAutogenerated=True)
                 labelSym.referenceCounter += 1
 
             elif self.popPointerInDataReference(currentVram) is not None:
-                if common.GlobalConfig.ADD_NEW_SYMBOLS:
-                    contextSym = self.addSymbol(currentVram, self.sectionType, isAutogenerated=True)
-                    contextSym.isMaybeString = self._stringGuesser(contextSym, localOffset)
+                contextSym = self.addSymbol(currentVram, self.sectionType, isAutogenerated=True)
+                contextSym.isMaybeString = self._stringGuesser(contextSym, localOffset)
 
             elif contextSym is not None:
                 contextSym.isMaybeString = self._stringGuesser(contextSym, localOffset)
 
-            contextSym = self.getSymbol(currentVram, tryPlusOffset=False)
+            # contextSym = self.getSymbol(currentVram, tryPlusOffset=False)
             if contextSym is not None:
                 self.symbolsVRams.add(currentVram)
 
@@ -128,6 +164,7 @@ class SectionRodata(SectionBase):
             sym = symbols.SymbolRodata(self.context, vrom, vromEnd, offset + self.inFileOffset, vram, words, self.segmentVromStart, self.overlayCategory)
             sym.parent = self
             sym.setCommentOffset(self.commentOffset)
+            sym.stringEncoding = self.stringEncoding
             sym.analyze()
             self.symbolList.append(sym)
 
@@ -135,7 +172,7 @@ class SectionRodata(SectionBase):
             if sym.inFileOffset % 16 == 0:
                 # Files are always 0x10 aligned
 
-                if previousSymbolWasLateRodata and not sym.contextSym.isLateRodata():
+                if previousSymbolWasLateRodata and not sym.contextSym.isLateRodata() and common.GlobalConfig.COMPILER == common.Compiler.IDO:
                     # late rodata followed by normal rodata implies a file split
                     self.fileBoundaries.append(sym.inFileOffset)
                 elif previousSymbolExtraPadding > 0:
