@@ -8,6 +8,7 @@ from __future__ import annotations
 import rabbitizer
 
 from ... import common
+from ... import elf32
 
 from . import SymbolText, analysis
 
@@ -45,7 +46,7 @@ class SymbolFunction(SymbolText):
 
         prevInstrOffset = instructionOffset - 4
         prevVram = self.getVramOffset(prevInstrOffset)
-        branchOffset = prevInstr.getGenericBranchOffset(prevVram)
+        branchOffset = prevInstr.getBranchOffsetGeneric()
         branch = prevInstrOffset + branchOffset
 
         if branch < 0:
@@ -54,7 +55,7 @@ class SymbolFunction(SymbolText):
 
         regsTracker = rabbitizer.RegistersTracker(trackedRegistersOriginal)
 
-        self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram)
+        self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, None, self.context.got)
 
         if instructionOffset in self.branchesTaken:
             return
@@ -65,7 +66,7 @@ class SymbolFunction(SymbolText):
             prevTargetInstr = self.instructions[branch//4 - 1]
             targetInstr = self.instructions[branch//4]
 
-            self.instrAnalyzer.processInstr(regsTracker, targetInstr, branch, self.getVramOffset(branch), prevTargetInstr)
+            self.instrAnalyzer.processInstr(regsTracker, targetInstr, branch, self.getVramOffset(branch), prevTargetInstr, self.context.got)
 
             if prevTargetInstr.isUnconditionalBranch():
                 return
@@ -96,7 +97,9 @@ class SymbolFunction(SymbolText):
                                 common.Utils.eprint(f"Warning. Jumptable referenced in reloc does not have '.rodata' as its name")
                             contextOffsetSym = self.context.addOffsetJumpTable(addressOffset, sectType)
                             contextOffsetSym.referenceCounter += 1
-                            relocSymbol.name = contextOffsetSym.name
+                            contextOffsetSym.referenceFunctions.add(self.contextSym)
+                            if contextOffsetSym.name is not None:
+                                relocSymbol.name = contextOffsetSym.name
                             self.instrAnalyzer.symbolInstrOffset[instructionOffset] = 0
                             if instructionOffset in self.instrAnalyzer.lowToHiDict:
                                 luiOffset = self.instrAnalyzer.lowToHiDict[instructionOffset]
@@ -145,7 +148,7 @@ class SymbolFunction(SymbolText):
                 return
 
             if not prevInstr.isBranchLikely() and not prevInstr.isUnconditionalBranch():
-                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr)
+                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr, self.context.got)
 
             # look-ahead symbol finder
             self._lookAheadSymbolFinder(instr, prevInstr, instructionOffset, regsTracker)
@@ -163,11 +166,13 @@ class SymbolFunction(SymbolText):
             branch = self.instrAnalyzer.branchTargetInstrOffsets[instrOffset]
             labelSym = self.addBranchLabel(targetBranchVram, isAutogenerated=True, symbolVrom=self.getVromOffset(branch))
             labelSym.referenceCounter += 1
+            labelSym.referenceFunctions.add(self.contextSym)
 
         # Function calls
         for targetVram in self.instrAnalyzer.funcCallInstrOffsets.values():
             funcSym = self.addFunction(targetVram, isAutogenerated=True)
             funcSym.referenceCounter += 1
+            funcSym.referenceFunctions.add(self.contextSym)
 
         if not self.isRsp and len(self.instrAnalyzer.funcCallOutsideRangesOffsets) > 0:
             self.isLikelyHandwritten = True
@@ -182,31 +187,51 @@ class SymbolFunction(SymbolText):
             if patchedAddress is not None:
                 symVram = patchedAddress
 
-            symType = self.instrAnalyzer.possibleSymbolTypes.get(symVram, None)
+            symAccess = self.instrAnalyzer.possibleSymbolTypes.get(symVram, None)
             contextSym = self.getSymbol(symVram)
             if contextSym is None:
-                if not common.GlobalConfig.ADD_NEW_SYMBOLS:
-                    continue
                 contextSym = self.addSymbol(symVram, isAutogenerated=True)
             else:
                 # TODO: do this in a less ugly way
                 if contextSym.address != symVram:
                     if contextSym.address % 4 != 0 or symVram % 4 != 0:
-                        if contextSym.getType() in {"u16", "s16", "u8", "u8"} or symType in {"u16", "s16", "u8", "u8"}:
+                        if contextSym.getType() in {"u16", "s16", "u8", "u8"} or (symAccess is not None and symAccess[0] in {rabbitizer.AccessType.BYTE, rabbitizer.AccessType.SHORT}):
                             if not (contextSym.getSize() > 4):
-                                if common.GlobalConfig.ADD_NEW_SYMBOLS:
+                                if contextSym.size is None or symVram >= contextSym.address + contextSym.size:
+                                    if symAccess is not None:
+                                        contextSym.setAccessTypeIfUnset(symAccess[0], symAccess[1])
                                     contextSym = self.addSymbol(symVram, isAutogenerated=True)
 
             contextSym.referenceCounter += 1
-            if symType is not None:
-                contextSym.setTypeIfUnset(symType)
+            contextSym.referenceFunctions.add(self.contextSym)
+            if symAccess is not None:
+                contextSym.setAccessTypeIfUnset(symAccess[0], symAccess[1])
 
         # Jump tables
         for targetVram in self.instrAnalyzer.jumpRegisterIntrOffset.values():
             self.addJumpTable(targetVram, isAutogenerated=True)
 
-        for instr in self.instructions:
-            instr.inHandwrittenFunction = self.isLikelyHandwritten
+        for gpLoadOffset in self.instrAnalyzer.gpLoads.keys():
+            gpSymbolAddress = self.instrAnalyzer.symbolHiInstrOffset.get(gpLoadOffset, None)
+            if gpSymbolAddress is not None:
+                # In case this is a paired %got symbol then tell the upper gp access the real address
+                self.instrAnalyzer.symbolInstrOffset[gpLoadOffset] = gpSymbolAddress
+            else:
+                # This $gp load was not paired with any other symbol
+                gpSymbolAddress = self.instrAnalyzer.symbolInstrOffset.get(gpLoadOffset)
+            if gpSymbolAddress is not None:
+                contextSym = self.getSymbol(gpSymbolAddress, tryPlusOffset=False)
+                if contextSym is not None:
+                    contextSym.isGot = True
+            elif common.GlobalConfig.GP_VALUE is not None:
+                gpLoadInstr = self.instrAnalyzer.gpLoads[gpLoadOffset]
+                gpSymbolAddress = common.GlobalConfig.GP_VALUE + gpLoadInstr.getProcessedImmediate()
+                self.instrAnalyzer.symbolInstrOffset[gpLoadOffset] = gpSymbolAddress
+                self.addSymbol(gpSymbolAddress, isAutogenerated=True)
+
+        if self.isLikelyHandwritten:
+            for instr in self.instructions:
+                instr.inHandwrittenFunction = self.isLikelyHandwritten
 
 
     def countExtraPadding(self) -> int:
@@ -302,50 +327,105 @@ class SymbolFunction(SymbolText):
         return was_updated
 
 
-    def generateHiLoStr(self, instr: rabbitizer.Instruction, symName: str) -> str:
+    def generateHiLoStr(self, instr: rabbitizer.Instruction, symName: str, symbol: common.ContextSymbol|None, relocInfo: common.ContextRelocInfo|None=None) -> str:
+        if relocInfo is not None:
+            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_HI16.value:
+                return f"%hi({symName})"
+            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_LO16.value:
+                return f"%lo({symName})"
+            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_GPREL16.value:
+                return f"%gp_rel({symName})"
+            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_GOT16.value:
+                return f"%got({symName})"
+            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_CALL16.value:
+                return f"%call16({symName})"
+            common.Utils.eprint(f"generateHiLoStr: Warning: Unhandled relocType '{relocInfo.relocType}'")
+
         if instr.canBeHi():
             return f"%hi({symName})"
 
         if instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
-            if instr.rt in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp} or not instr.modifiesRt():
+            if common.GlobalConfig.PIC:
+                if symbol is not None:
+                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
+                        return f"%call16({symName})"
+                    elif symbol.isGot:
+                        return f"%got({symName})"
+            if not common.GlobalConfig.PIC:
                 return f"%gp_rel({symName})"
 
         return f"%lo({symName})"
 
+    def generateHiLoConstantStr(self, constantValue: int, currentInstr: rabbitizer.Instruction, loInstr: rabbitizer.Instruction|None) -> str|None:
+        if loInstr is None:
+            if currentInstr.canBeHi():
+                return f"(0x{constantValue:X} >> 16)"
+            return None
+
+        if loInstr.canBeLo():
+            if loInstr.isUnsigned():
+                if currentInstr.canBeHi():
+                    return f"(0x{constantValue:X} >> 16)"
+                if currentInstr.canBeLo():
+                    return f"(0x{constantValue:X} & 0xFFFF)"
+                return None
+            else:
+                hiHalf = constantValue > 16
+                loHalf = constantValue & 0xFFFF
+                if loHalf < 0x8000:
+                    # positive lo half
+                    if currentInstr.canBeHi():
+                        return f"(0x{constantValue:X} >> 16)"
+                    if currentInstr.canBeLo():
+                        return f"(0x{constantValue:X} & 0xFFFF)"
+                else:
+                    # negative lo half
+                    # loHalf = rabbitizer.Utils.from2Complement(loHalf, 16)
+                    pass
+        return None
+
+
     def getImmOverrideForInstruction(self, instr: rabbitizer.Instruction, instructionOffset: int) -> str|None:
         if len(self.context.relocSymbols[self.sectionType]) > 0:
             # Check possible symbols using reloc information (probably from a .o elf file)
             possibleImmOverride = self.context.getRelocSymbol(self.inFileOffset + instructionOffset, self.sectionType)
             if possibleImmOverride is not None:
                 auxOverride = possibleImmOverride.getName()
-                if instr.isIType():
+                if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
                     if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
                         addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
                         auxOverride = possibleImmOverride.getNamePlusOffset(addressOffset)
 
-                    auxOverride = self.generateHiLoStr(instr, auxOverride)
+                    auxOverride = self.generateHiLoStr(instr, auxOverride, None, relocInfo=possibleImmOverride)
                 return auxOverride
 
         if instr.isBranch() or instr.isUnconditionalBranch():
             if not common.GlobalConfig.IGNORE_BRANCHES:
-                branchOffset = instr.getGenericBranchOffset(self.getVramOffset(instructionOffset))
+                branchOffset = instr.getBranchOffsetGeneric()
                 targetBranchVram = self.getVramOffset(instructionOffset + branchOffset)
                 labelSymbol = self.getSymbol(targetBranchVram, tryPlusOffset=False)
                 if labelSymbol is not None:
                     return labelSymbol.getName()
 
-        elif instr.isIType():
+        elif instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+            # .cpload directive is meant to use the `_gp_disp` pseudo-symbol
+            if instructionOffset in self.instrAnalyzer.cploadOffsets:
+                return self.generateHiLoStr(instr, "_gp_disp", None)
+
             if not self.pointersRemoved and instructionOffset in self.instrAnalyzer.symbolInstrOffset:
                 address = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
 
                 if address in self.context.bannedSymbols:
                     return None
 
+                loInstr = instr
                 instrVram = self.getVramOffset(instructionOffset)
                 if instr.canBeHi():
                     # we need to get the address of the lo instruction to get the patch
                     if instructionOffset in self.instrAnalyzer.hiToLowDict:
-                        instrVram = self.getVramOffset(self.instrAnalyzer.hiToLowDict[instructionOffset])
+                        loOffset = self.instrAnalyzer.hiToLowDict[instructionOffset]
+                        instrVram = self.getVramOffset(loOffset)
+                        loInstr = self.instructions[loOffset // 4]
 
                 # Check for user-defined symbol patches
                 patchedAddress = self.getLoPatch(instrVram)
@@ -355,31 +435,45 @@ class SymbolFunction(SymbolText):
                     symbol = self.getSymbol(address, tryPlusOffset=True)
 
                 if symbol is not None:
-                    return self.generateHiLoStr(instr, symbol.getSymbolPlusOffset(address))
+                    symName = symbol.getSymbolPlusOffset(address)
+                    if common.GlobalConfig.PIC:
+                        if instructionOffset not in self.instrAnalyzer.gpLoads:
+                            if address in self.context.got.globalsTable:
+                                # addend = instr.getProcessedImmediate()
+                                # if addend < 0:
+                                #     symName = f"{symName} - 0x{-addend:X}"
+                                # elif addend > 0:
+                                #     symName = f"{symName} + 0x{addend:X}"
+                                # # do nothing if addend is zero
+                                return None
+
+                    return self.generateHiLoStr(instr, symName, symbol)
+                return self.generateHiLoConstantStr(address, instr, loInstr)
 
             elif instructionOffset in self.instrAnalyzer.constantInstrOffset:
                 constant = self.instrAnalyzer.constantInstrOffset[instructionOffset]
 
                 symbol = self.getConstant(constant)
                 if symbol is not None:
-                    return self.generateHiLoStr(instr, symbol.getName())
+                    return self.generateHiLoStr(instr, symbol.getName(), symbol)
 
                 # Pretend this pair is a constant
+                loInstr = instr
                 if instr.canBeHi():
                     loInstr = self.instructions[self.instrAnalyzer.hiToLowDict[instructionOffset] // 4]
-                    if loInstr.canBeLo() and loInstr.isUnsigned():
-                        return f"(0x{constant:X} >> 16)"
-                elif instr.canBeLo() and instr.isUnsigned():
-                    return f"(0x{constant:X} & 0xFFFF)"
+
+                generatedStr = self.generateHiLoConstantStr(constant, instr, loInstr)
+                if generatedStr is not None:
+                    return generatedStr
 
                 if common.GlobalConfig.SYMBOL_FINDER_FILTERED_ADDRESSES_AS_HILO:
-                    return self.generateHiLoStr(instr, f"0x{constant:X}")
+                    return self.generateHiLoStr(instr, f"0x{constant:X}", None)
 
-            elif instr.canBeHi():
+            if instr.canBeHi():
                 # Unpaired LUI
-                return f"(0x{instr.getImmediate()<<16:X} >> 16)"
+                return self.generateHiLoConstantStr(instr.getProcessedImmediate()<<16, instr, None)
 
-        elif instr.isJType():
+        elif instr.isJumpWithAddress():
             possibleOverride = self.getSymbol(instr.getInstrIndexAsVram(), tryPlusOffset=False)
             if possibleOverride is not None:
                 return possibleOverride.getName()
@@ -398,7 +492,7 @@ class SymbolFunction(SymbolText):
         if labelSym is None and len(self.context.offsetSymbols[self.sectionType]) > 0:
             labelSym = self.context.getOffsetSymbol(self.inFileOffset+instructionOffset, common.FileSectionType.Text)
 
-        if labelSym is None or labelSym.overlayCategory != self.overlayCategory:
+        if labelSym is None or labelSym.overlayCategory != self.overlayCategory or labelSym.isGotLocal:
             return ""
 
         labelSym.isDefined = True
@@ -412,6 +506,40 @@ class SymbolFunction(SymbolText):
             return label
         return labelSym.getName() + ":" + common.GlobalConfig.LINE_ENDS
 
+    def _emitInstruction(self, instr: rabbitizer.Instruction, instructionOffset: int, wasLastInstABranch: bool) -> str:
+        immOverride = self.getImmOverrideForInstruction(instr, instructionOffset)
+        comment = self.generateAsmLineComment(instructionOffset, instr.getRaw())
+        extraLJust = 0
+
+        if wasLastInstABranch:
+            extraLJust = -1
+            comment += " "
+
+        line = instr.disassemble(immOverride, extraLJust=extraLJust)
+
+        return f"{comment}  {line}{common.GlobalConfig.LINE_ENDS}"
+
+
+    def _emitCpload(self, instr: rabbitizer.Instruction, instructionOffset: int, wasLastInstABranch: bool) -> str:
+        output = ""
+
+        cpload = self.instrAnalyzer.cploads.get(instructionOffset)
+        if cpload is not None:
+            hiInstr = self.instructions[cpload.hiOffset//4]
+            loInstr = self.instructions[cpload.loOffset//4]
+            gpDisp = hiInstr.getProcessedImmediate() << 16
+            gpDisp += loInstr.getProcessedImmediate()
+            output += f"# _gp_disp: 0x{gpDisp:X}{common.GlobalConfig.LINE_ENDS}"
+            if common.GlobalConfig.EMIT_CPLOAD:
+                assert cpload.reg is not None
+                output += f".set noreorder; .cpload ${cpload.reg.name}; # .set reorder" + common.GlobalConfig.LINE_ENDS
+            else:
+                output += self._emitInstruction(instr, instructionOffset, wasLastInstABranch)
+        else:
+            if not common.GlobalConfig.EMIT_CPLOAD:
+                output += self._emitInstruction(instr, instructionOffset, wasLastInstABranch)
+            # don't emit the other instructions which are part of .cpload if the directive was emitted
+        return output
 
     def disassemble(self) -> str:
         output = ""
@@ -436,18 +564,14 @@ class SymbolFunction(SymbolText):
         wasLastInstABranch = False
         instructionOffset = 0
         for instr in self.instructions:
-            immOverride = self.getImmOverrideForInstruction(instr, instructionOffset)
-            comment = self.generateAsmLineComment(instructionOffset, instr.getRaw())
-            extraLJust = 0
-
-            if wasLastInstABranch:
-                extraLJust = -1
-                comment += " "
-
-            line = instr.disassemble(immOverride, extraLJust=extraLJust)
-
             label = self.getLabelForOffset(instructionOffset)
-            output += f"{label}{comment}  {line}" + common.GlobalConfig.LINE_ENDS
+            output += label
+
+            isCpload = instructionOffset in self.instrAnalyzer.cploadOffsets
+            if isCpload:
+                output += self._emitCpload(instr, instructionOffset, wasLastInstABranch)
+            else:
+                output += self._emitInstruction(instr, instructionOffset, wasLastInstABranch)
 
             wasLastInstABranch = instr.hasDelaySlot()
             instructionOffset += 4
