@@ -5,11 +5,19 @@
 
 from __future__ import annotations
 
+import dataclasses
 import rabbitizer
 
 from .... import common
 
 
+@dataclasses.dataclass
+class CploadInfo:
+    hiOffset: int
+    loOffset: int
+    adduOffset: int|None = None
+    reg: rabbitizer.Enum|None = None
+
 class InstrAnalyzer:
     def __init__(self, funcVram: int) -> None:
         self.funcVram = funcVram
@@ -58,7 +66,8 @@ class InstrAnalyzer:
 
         self.symbolInstrOffset: dict[int, int] = dict()
 
-        self.possibleSymbolTypes: dict[int, str] = dict()
+        self.possibleSymbolTypes: dict[int, tuple[rabbitizer.Enum, bool]] = dict()
+        "key: address, value: (<rabbitizer.AccessType>, unsignedMemoryAccess)"
 
         # %hi/%lo pairing
         self.hiToLowDict: dict[int, int] = dict()
@@ -70,13 +79,22 @@ class InstrAnalyzer:
 
         self.nonLoInstrOffsets: set[int] = set()
 
+        self.gpLoads: dict[int, rabbitizer.Instruction] = dict()
+
+        self.unpairedCploads: list[CploadInfo] = list()
+        "cploads which are not yet fully paired"
+        self.cploadOffsets: set[int] = set()
+        "Offset of every cpload instruction"
+        self.cploads: dict[int, CploadInfo] = dict()
+        "Completed cpload, key: offset of last instruction of the cpload"
+
 
     def processBranch(self, instr: rabbitizer.Instruction, instrOffset: int, currentVram: int) -> None:
         if instrOffset in self.branchInstrOffsets:
             # Already processed
             return
 
-        branchOffset = instr.getGenericBranchOffset(currentVram)
+        branchOffset = instr.getBranchOffsetGeneric()
         branch = instrOffset + branchOffset
         targetBranchVram = self.funcVram + branch
 
@@ -102,8 +120,8 @@ class InstrAnalyzer:
 
 
     def processConstant(self, regsTracker: rabbitizer.RegistersTracker, luiInstr: rabbitizer.Instruction, luiOffset: int, lowerInstr: rabbitizer.Instruction, lowerOffset: int) -> int|None:
-        upperHalf = luiInstr.getImmediate() << 16
-        lowerHalf = lowerInstr.getImmediate()
+        upperHalf = luiInstr.getProcessedImmediate() << 16
+        lowerHalf = lowerInstr.getProcessedImmediate()
         constant = upperHalf | lowerHalf
 
         self.referencedConstants.add(constant)
@@ -121,22 +139,22 @@ class InstrAnalyzer:
         return constant
 
 
-    def pairHiLo(self, luiInstr: rabbitizer.Instruction|None, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int) -> int|None:
+    def pairHiLo(self, hiValue: int|None, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int, got: common.GlobalOffsetTable, otherIsGpGot: bool) -> int|None:
         # lui being None means this symbol is a $gp access
-        assert (luiInstr is None and luiOffset is None) or (luiInstr is not None and luiOffset is not None)
+        assert (hiValue is None and luiOffset is None) or (hiValue is not None and luiOffset is not None)
 
-        lowerHalf = rabbitizer.Utils.from2Complement(lowerInstr.getImmediate(), 16)
+        lowerHalf = lowerInstr.getProcessedImmediate()
 
         if lowerOffset in self.symbolLoInstrOffset:
             # This %lo has been processed already
 
             # Check the other lui has the same immediate value as this one, and reject the pair if it doesn't
-            if luiInstr is not None:
+            if hiValue is not None:
                 otherLuiOffset = self.lowToHiDict.get(lowerOffset, None)
                 if otherLuiOffset is not None:
                     otherLuiInstr = self.luiInstrs.get(otherLuiOffset, None)
                     if otherLuiInstr is not None:
-                        if luiInstr.getImmediate() != otherLuiInstr.getImmediate():
+                        if hiValue != otherLuiInstr.getProcessedImmediate() << 16:
                             return None
 
             if common.GlobalConfig.COMPILER == common.Compiler.IDO:
@@ -144,7 +162,7 @@ class InstrAnalyzer:
                 return self.symbolLoInstrOffset[lowerOffset]
 
             elif common.GlobalConfig.COMPILER in {common.Compiler.GCC, common.Compiler.SN64}:
-                if luiOffset is None or luiInstr is None:
+                if luiOffset is None or hiValue is None:
                     return None
 
                 if self.hiToLowDict.get(luiOffset, None) == lowerOffset and self.lowToHiDict.get(lowerOffset, None) == luiOffset:
@@ -164,7 +182,7 @@ class InstrAnalyzer:
                     # Make an exception if the lower instruction is just after the LUI
                     pass
                 else:
-                    upperHalf = luiInstr.getImmediate() << 16
+                    upperHalf = hiValue
                     address = upperHalf + lowerHalf
                     if address == self.symbolLoInstrOffset[lowerOffset]:
                         # Make an exception if the resulting address is the same
@@ -172,20 +190,29 @@ class InstrAnalyzer:
                     else:
                         return self.symbolLoInstrOffset[lowerOffset]
 
-        if luiInstr is None and common.GlobalConfig.GP_VALUE is None:
+        if hiValue is None and common.GlobalConfig.GP_VALUE is None:
             # Trying to pair a gp relative offset, but we don't know the gp address
             return None
 
-        if luiInstr is not None:
-            upperHalf = luiInstr.getImmediate() << 16
+        if hiValue is not None:
+            upperHalf = hiValue
+            if otherIsGpGot and upperHalf in got.globalsTable:
+                lowerHalf = 0
         else:
             assert common.GlobalConfig.GP_VALUE is not None
             upperHalf = common.GlobalConfig.GP_VALUE
 
+            if got.tableStart is not None:
+                return got.getAddress(upperHalf + lowerHalf)
+
         return upperHalf + lowerHalf
 
 
     def processSymbol(self, address: int, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int) -> int|None:
+        if address <= 0:
+            # PS2 seems to do LUI/ADDIU pairs for 0 and -1 for some reason, filter them out
+            return None
+
         # filter out stuff that may not be a real symbol
         filterOut = common.GlobalConfig.SYMBOL_FINDER_FILTER_LOW_ADDRESSES and address < 0x80000000
         filterOut |= common.GlobalConfig.SYMBOL_FINDER_FILTER_HIGH_ADDRESSES and address >= 0xC0000000
@@ -229,12 +256,17 @@ class InstrAnalyzer:
         return address
 
     def processSymbolType(self, address: int, instr: rabbitizer.Instruction) -> None:
-        instrType = instr.mapInstrToType()
-        if instrType is None:
+        accessType = instr.getAccessType()
+        unsignedMemoryAccess = instr.doesUnsignedMemoryAccess()
+        if accessType == rabbitizer.AccessType.INVALID:
             return
 
+        if accessType == rabbitizer.AccessType.WORD:
+            if not unsignedMemoryAccess:
+                return
+
         if address not in self.possibleSymbolTypes:
-            self.possibleSymbolTypes[address] = instrType
+            self.possibleSymbolTypes[address] = (accessType, unsignedMemoryAccess)
 
     def processSymbolDereferenceType(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int) -> None:
         address = regsTracker.getAddressIfCanSetType(instr, instrOffset)
@@ -244,7 +276,7 @@ class InstrAnalyzer:
         self.processSymbolType(address, instr)
 
 
-    def symbolFinder(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, prevInstr: rabbitizer.Instruction|None, instrOffset: int) -> None:
+    def symbolFinder(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, prevInstr: rabbitizer.Instruction|None, instrOffset: int, got: common.GlobalOffsetTable) -> None:
         if instr.canBeHi():
             if prevInstr is None:
                 regsTracker.processLui(instr, instrOffset)
@@ -253,6 +285,11 @@ class InstrAnalyzer:
             self.luiInstrs[instrOffset] = instr
             return
 
+        instrDoesGpLoad = False
+        if instr.doesLoad() and instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
+            regsTracker.processGpLoad(instr, instrOffset)
+            instrDoesGpLoad = True
+
         if not instr.canBeLo():
             return
 
@@ -270,25 +307,38 @@ class InstrAnalyzer:
         if instrOffset in self.nonLoInstrOffsets:
             return
 
-        luiOffset, isGp, shouldProcess = regsTracker.getLuiOffsetForLo(instr, instrOffset)
-        if not shouldProcess:
+        pairingInfo = regsTracker.preprocessLoAndGetInfo(instr, instrOffset)
+        if not pairingInfo.shouldProcess:
             if regsTracker.hasLoButNoHi(instr):
                 self.nonLoInstrOffsets.add(instrOffset)
             return
 
-        if isGp:
+        if pairingInfo.isGpGot and not common.GlobalConfig.PIC:
+            return
+
+        upperHalf: int|None = pairingInfo.value
+        luiOffset = pairingInfo.instrOffset
+        if pairingInfo.isGpRel:
+            upperHalf = None
             luiOffset = None
 
-        luiInstr = None
         if luiOffset is not None:
-            luiInstr = self.luiInstrs.get(luiOffset, None)
-            if luiInstr is None:
-                return
+            luiInstr = self.luiInstrs.get(luiOffset)
+            if luiInstr is not None:
+                if luiInstr.rt in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
+                    if instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp} and instr.rt in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
+                        # cpload
+                        self.unpairedCploads.append(CploadInfo(luiOffset, instrOffset))
+                        # early return to avoid counting this pairing as a symbol
+                        return
 
-        address = self.pairHiLo(luiInstr, luiOffset, instr, instrOffset)
+        address = self.pairHiLo(upperHalf, luiOffset, instr, instrOffset, got, pairingInfo.isGpGot)
         if address is None:
             return
 
+        if instrDoesGpLoad:
+            self.gpLoads[instrOffset] = instr
+
         address = self.processSymbol(address, luiOffset, instr, instrOffset)
         if address is not None:
             regsTracker.processLo(instr, address, instrOffset)
@@ -304,20 +354,32 @@ class InstrAnalyzer:
             self.referencedVrams.add(address)
 
 
-    def processInstr(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int, currentVram: int, prevInstr: rabbitizer.Instruction|None=None) -> None:
+    def processInstr(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int, currentVram: int, prevInstr: rabbitizer.Instruction|None, got: common.GlobalOffsetTable) -> None:
         if instr.isBranch() or instr.isUnconditionalBranch():
             self.processBranch(instr, instrOffset, currentVram)
 
-        elif instr.isJType():
+        elif instr.isJumpWithAddress():
             self.processFuncCall(instr, instrOffset)
 
-        elif instr.isIType():
-            self.symbolFinder(regsTracker, instr, prevInstr, instrOffset)
+        elif instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+            self.symbolFinder(regsTracker, instr, prevInstr, instrOffset, got)
             self.processSymbolDereferenceType(regsTracker, instr, instrOffset)
 
         elif instr.isJrNotRa():
             self.processJumpRegister(regsTracker, instr, instrOffset)
 
+        elif instr.uniqueId == rabbitizer.InstrId.cpu_addu:
+            # special check for .cpload
+            if len(self.unpairedCploads) > 0:
+                if instr.rd in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp} and instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
+                    cpload = self.unpairedCploads.pop()
+                    cpload.adduOffset = instrOffset
+                    cpload.reg = instr.rt
+                    self.cploadOffsets.add(cpload.hiOffset)
+                    self.cploadOffsets.add(cpload.loOffset)
+                    self.cploadOffsets.add(instrOffset)
+                    self.cploads[instrOffset] = cpload
+
         regsTracker.overwriteRegisters(instr, instrOffset)
 
 
@@ -357,9 +419,9 @@ class InstrAnalyzer:
                 # print(f"C  {self.constantsPerInstruction[instructionOffset]:8X}", luiInstr)
                 pass
             else:
-                if common.GlobalConfig.SYMBOL_FINDER_FILTER_LOW_ADDRESSES and luiInstr.getImmediate() < 0x8000: # filter out stuff that may not be a real symbol
+                if common.GlobalConfig.SYMBOL_FINDER_FILTER_LOW_ADDRESSES and luiInstr.getProcessedImmediate() < 0x8000: # filter out stuff that may not be a real symbol
                     continue
-                if common.GlobalConfig.SYMBOL_FINDER_FILTER_HIGH_ADDRESSES and luiInstr.getImmediate() >= 0xC000: # filter out stuff that may not be a real symbol
+                if common.GlobalConfig.SYMBOL_FINDER_FILTER_HIGH_ADDRESSES and luiInstr.getProcessedImmediate() >= 0xC000: # filter out stuff that may not be a real symbol
                     continue
 
                 # print(f"{currentVram:06X} ", end="")
